
import gc
import gzip
import resource
import os
import simplejson
import subprocess
import sys
import time


# My machine gets really unhappy if you hit swap, so avoid it, 3GB seems to be
# usable, but not great, so back off a little bit.
gb = 1024*1024*1024
max_mem = int(2.5*gb)
resource.setrlimit(resource.RLIMIT_AS, (max_mem, max_mem))


def zcat(in_f):
    z_in = gzip.GzipFile(fileobj=in_f, mode='rb')
    try:
        for line in z_in:
            yield line
    finally:
        z_in.close()


def spawn_usage():
    try:
        subprocess.call('grep --color VmPeak /proc/%d/status' % (os.getpid(),),
                        shell=True)
    except:
        sys.stderr.write("Failed to spawn grep\n")


def main(args):
    import optparse
    p = optparse.OptionParser(usage='%prog [options]', version='%prog v0.1')
    p.add_option('--verbose', action='store_true', help='Be chatty')
    p.add_option('--keep', action='store_true',
        help='Keep the data in memory while parsing, appending it to a list')
    p.add_option('--disable-speedups', action='store_true',
        help='Disable the simplejson C extensions.')
    (opts,args) = p.parse_args(args)

    to_close = []
    if len(args) >= 1:
        in_f = open(args[0], 'rb')
        to_close.append(in_f)
    else:
        in_f = sys.stdin
    if opts.disable_speedups:
        simplejson._toggle_speedups(False)
    all_res = []
    t = time.time()
    try:
        count = 0
        for line in zcat(in_f):
            data = simplejson.loads(line)
            if opts.keep:
                all_res.append(data)
            count += 1
            if count % 10000 == 0:
                delta = time.time() - t
                sys.stdout.write("Read %d lines in %.3fs\r" % (count, delta))
                sys.stdout.flush()
    finally:
        del all_res[:]
        all_res = []
        gc.collect()
        for x in to_close:
            x.close()
        delta = time.time() - t
        print "Read %d lines in %.3fs" % (count, delta)
        rusage = resource.getrusage(resource.RUSAGE_SELF)
        print "Peak Mem: %.1fMiB" % (rusage.ru_maxrss / 1024.,)
        spawn_usage()


if __name__ == '__main__':
    main(sys.argv[1:])
